#!/usr/bin/python

import string
import sys
import os
import libconf
import libfbase
import libfido

USRIF_CONFIG = '/usr/local/etc/u-srif/u-srif.conf'
MSG_FROM = 'FREQ manager'
MSG_SUBJ = 'FREQ results'

class freq_report(libfido.message):
	def write_packet(self, pktname):
		self.packet.addr_orig = self.addr_orig
		self.packet.addr_dest = self.addr_dest
		self.packet.write(pktname)
	
	def add_file(self, name, size, desc):
		text = libfbase.format_file_info(name, size, desc)
		self.append_text(text)
	
	def __init__(self):
		libfido.message.__init__(self)
		self.packet  = libfido.packet()
		self.packet.add_message(self)

class srif_file:
	def read_req_list(self):
		yield = []
		fp = open(self.requestlist, 'r')
		line = fp.readline()
		while line:
			line = string.strip(line)
			yield.append(line)
			line = fp.readline()
		fp.close()
		return yield

	def write_resp_list(self, list):
		fp = open(self.responselist, 'w')
		for file in list:
			fp.write('+' + file + '\n')
		fp.close()
	
	def read(self, name):
		fp = open(name, 'r')
		line = fp.readline()
		while line:
			line = string.strip(line)
			args = string.split(line, None, 1)
			if len(args) == 2:
				if string.lower(args[0]) == 'sysop':
					self.sysop = args[1]
				if string.lower(args[0]) == 'aka':
					self.aka = args[1]
				elif string.lower(args[0]) == 'baud':
					self.baud = args[1]
				elif string.lower(args[0]) == 'requestlist':
					self.requestlist = args[1]
				elif string.lower(args[0]) == 'responselist':
					self.responselist = args[1]
				elif string.lower(args[0]) == 'remotestatus':
					self.remotestatus = args[1]
				elif string.lower(args[0]) == 'systemstatus':
					self.systemstatus = args[1]
				elif string.lower(args[0]) == 'site':
					self.site = args[1]
				elif string.lower(args[0]) == 'callerid':
					self.callerid = args[1]
				elif string.lower(args[0]) == 'password':
					self.password = args[1]
				else:
					print "skipping keyword", args[0], "in SRIF"
			line = fp.readline()
		fp.close()
		
	def __init__(self, name):
		self.sysop = ''
		self.aka = ''
		self.baud = 0
		self.requestlist = ''
		self.responselist = ''
		self.remotestatus = ''
		self.systemstatus = ''
		self.site = ''
		self.callerid = ''
		self.password = ''
		self.read(name)
	
	def remote_addr(self):
		return self.aka
	
	def isprotected(self):
		if string.lower(self.remotestatus) == 'protected':
			return 1
		return 0
	
	def islisted(self):
		if string.lower(self.systemstatus) == 'listed':
			return 1
		return 0

def append_new_file(fileslist, file):
	TotalFiles = TotalFiles + 1
	TotalSize = TotalSize + file.size
	fileslist.append(file.fullname)

##############################
# The main program starts here

# Global variables
TotalFiles = 0
TotalSize = 0
TotalReqs = 0
ResponseList = []
Report = freq_report()

if len(sys.argv) <> 2:
	print 'usage: u-srif <srif file name>'
	sys.exit(1)

# Read configuration
conf = libconf.config(USRIF_CONFIG)

# Read SRIF files
Srif = srif_file(sys.argv[1])

# XXX
Report.newmsg('2:5020/2120', MSG_FROM, Srif.aka, Srif.sysop, MSG_SUBJ)
Report.append_line('')
Report.append_text(libfbase.file_info_header)

FBase = libfbase.filebase(conf.spool_dir)
FBase.open('r')
for req in Srif.read_req_list():
	TotalReqs = TotalReqs + 1
	files_list = FBase.get(req)
	if files_list == None:
		Report.add_file(req, -1, 'File not found')
		continue
	for file in files_list:
		file.stat()
		TotalFiles = TotalFiles + 1
		TotalSize = TotalSize + file.size
		Report.add_file(file.name, file.size, file.desc)
		ResponseList.append(file.fullname)
FBase.close()

# Append overall statistics
Report.append_line('')
Report.append_line('Total %d file(s) (%d bytes), %d file(s) were requested'\
                   % (TotalFiles, TotalSize, TotalReqs))

# Append the FREQs policy if specified
if conf.freq_policy:
	Report.append_line('')
	Report.append_file(conf.freq_policy)

# Add empty line to the report
Report.append_line('')

# Create netmail packet with the FREQ report
pktname = '/var/tmp/12345678.pkt' # XXX
Report.write_packet(pktname)

# Add packet file to the response files list
ResponseList.append(pktname)

# Dump reponse list
Srif.write_resp_list(ResponseList)

sys.exit(0)

