/*
 *	binkleyforce -- unix FTN mailer project
 *	
 *	Copyright (c) 1998-2000 Alexander Belkin, 2:5020/1398.11
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	$Id: prot_yoohoo.c,v 1.4 2000/06/19 19:16:40 adb Exp $
 */

#include "includes.h"
#include "confread.h"
#include "logger.h"
#include "util.h"
#include "session.h"

static char *yoohoo_putword(char *buf, int val)
{
	buf[0] = ( ((unsigned int) val)       ) & 0xff;
	buf[1] = ( ((unsigned int) val) >> 8  ) & 0xff;
	
	return buf + 2;
}

static int yoohoo_getword(const char *buf)
{
	return ( (unsigned int) ((unsigned char) buf[0])      )
	     | ( (unsigned int) ((unsigned char) buf[1]) << 8 );
}

/*
 * Put data from 'hello' structure into the buffer 'buf'
 * 'buf' must be at least 128 bytes length!
 */
static void yoohoo_puthello(char *buf, s_yoohoo *hello)
{
	char *p = buf;
	
	memset(buf, '\0', YOOHOO_HELLOLEN);
	
	*p++ = hello->signal & 0xff;                 /* Single */
	p = yoohoo_putword(p, hello->hello_version); /* Hello version */
	p = yoohoo_putword(p, hello->product);       /* Product code */
	p = yoohoo_putword(p, hello->version_maj);   /* Major version */
	p = yoohoo_putword(p, hello->version_min);   /* Minor version */
	
	strnxcpy(p, hello->name, 60);                /* Node name */
	
	/*
	 * Add domain after the end of 'Node name'
	 */
	if( hello->address.domain && *hello->address.domain )
	{
		char *q;
		if( strlen(hello->name) + strlen(hello->address.domain) > 57 )
		{
			if( strlen(hello->address.domain) < 60 )
				q = p + (60 - strlen(hello->address.domain));
			else
				q = p;
			
			*q++ = '\0';
		}
		else
		{
			q = p + strlen(hello->name) + 1;
		}
		strnxcpy(q, hello->address.domain, 60-(q-p));
	}
	p = p + 60;

	p = strnxcpy(p, hello->sysop, 20) + 20;      /* SysOp name */
	p = yoohoo_putword(p, hello->address.zone);  /* Zone */
	p = yoohoo_putword(p, hello->address.net);   /* Net */
	p = yoohoo_putword(p, hello->address.node);  /* Net */
	p = yoohoo_putword(p, hello->address.point); /* Point */
	p = strncpy(p, hello->password, 8) + 8;      /* Session password */
	p = p + 8;                                   /* Reserved 8 bytes */
	p = yoohoo_putword(p, hello->capabilities);  /* Capabilities */
	p = p + 12;                                  /* Reserved 12 bytes */

	ASSERT((p - buf) == YOOHOO_HELLOLEN);
}

static void yoohoo_gethello(s_yoohoo *hello, const char *buf)
{
	memset(hello, '\0', sizeof(s_yoohoo));
	
	hello->signal        = (int)(buf[0] & 0xff);
	hello->hello_version = yoohoo_getword(buf+1);
	hello->product       = yoohoo_getword(buf+3);
	hello->version_maj   = yoohoo_getword(buf+5);
	hello->version_min   = yoohoo_getword(buf+7);
	strnxcpy(hello->name,  buf+9, MIN(sizeof(hello->name),60+1));
	strnxcpy(hello->sysop, buf+69, MIN(sizeof(hello->sysop),20+1));
	hello->address.zone  = yoohoo_getword(buf+91);
	hello->address.net   = yoohoo_getword(buf+93);
	hello->address.node  = yoohoo_getword(buf+95);
	hello->address.point = yoohoo_getword(buf+97);
	strnxcpy(hello->password, buf+99, MIN(sizeof(hello->password),8+1));
	/* Reserved 8 bytes */
	hello->capabilities  = yoohoo_getword(buf+115);
	/* Reserved 12 bytes */
}
